@@ -100,7 +100,7 @@ class TestWriter(unittest.TestCase):
 
         for l, r in zip(records, reader2):
             self.assertEquals(l.samples, r.samples)
-            
+
 class TestRecord(unittest.TestCase):
 
     def test_num_calls(self):
@@ -160,7 +160,7 @@ class TestCall(unittest.TestCase):
     def test_phased(self):
         reader = vcf.Reader(fh('example.vcf'))
         for var in reader:
-            phases = [s.phased for s in var.samples] 
+            phases = [s.phased for s in var.samples]
             if var.POS == 14370:
                 self.assertEqual([True, True, False], phases)
             if var.POS == 17330:
@@ -186,7 +186,7 @@ class TestCall(unittest.TestCase):
                 self.assertEqual(['T|T', 'T|T', 'T/T'], gt_bases)
             elif var.POS == 1234567:
                 self.assertEqual([None, 'GTCT/GTACT', 'G/G'], gt_bases)
-                
+
     def test_gt_types(self):
         reader = vcf.Reader(fh('example.vcf'))
         for var in reader:
@@ -201,18 +201,18 @@ class TestCall(unittest.TestCase):
                 self.assertEqual([0,0,0], gt_types)
             elif var.POS == 1234567:
                 self.assertEqual([None,1,2], gt_types)
-                
+
 class TestTabix(unittest.TestCase):
 
     def setUp(self):
         self.reader = vcf.Reader(fh('tb.vcf.gz'))
 
-    def testFetch(self):
-        lines = list(self.reader.fetch('20', 14370-1, 14370+1))
+    def testFetchRange(self):
+        lines = list(self.reader.fetch('20', 14370, 14370))
         self.assertEquals(len(lines), 1)
         self.assertEqual(lines[0].POS, 14370)
 
-        lines = list(self.reader.fetch('20', 14370-1, 17330+1))
+        lines = list(self.reader.fetch('20', 14370, 17330))
         self.assertEquals(len(lines), 2)
         self.assertEqual(lines[0].POS, 14370)
         self.assertEqual(lines[1].POS, 17330)
@@ -221,6 +221,15 @@ class TestTabix(unittest.TestCase):
         lines = list(self.reader.fetch('20', 1110695, 1234567))
         self.assertEquals(len(lines), 3)
 
+    def testFetchSite(self):
+        site = self.reader.fetch('20', 14370)
+        assert site.POS == 14370
+
+        site = self.reader.fetch('20', 14369)
+        assert site is None
+
+
+
 
 class TestOpenMethods(unittest.TestCase):
 
@@ -248,16 +257,22 @@ class TestFilter(unittest.TestCase):
 
 
     def testApplyFilter(self):
-        out = commands.getoutput('python vcf_filter.py --site-quality 30 test/example.vcf site_quality')
+        s, out = commands.getstatusoutput('python vcf_filter.py --site-quality 30 test/example.vcf sq')
         #print out
+        assert s == 0
         buf = StringIO()
         buf.write(out)
         buf.seek(0)
+
+        print buf.getvalue()
         reader = vcf.Reader(buf)
 
+
         # check filter got into output file
         assert 'sq30' in reader.filters
 
+        print reader.filters
+
         # check sites were filtered
         n = 0
         for r in reader:
@@ -270,17 +285,26 @@ class TestFilter(unittest.TestCase):
 
 
     def testApplyMultipleFilters(self):
-        out = commands.getoutput('python vcf_filter.py --site-quality 30 '
-        '--genotype-quality 50 test/example.vcf site_quality min_genotype_quality')
+        s, out = commands.getstatusoutput('python vcf_filter.py --site-quality 30 '
+        '--genotype-quality 50 test/example.vcf sq mgq')
+        assert s == 0
         #print out
         buf = StringIO()
         buf.write(out)
         buf.seek(0)
         reader = vcf.Reader(buf)
 
+        print reader.filters
+
         assert 'mgq50' in reader.filters
         assert 'sq30' in reader.filters
 
+class TestRegression(unittest.TestCase):
+
+    def test_issue_16(self):
+        reader = vcf.Reader(fh('issue-16.vcf'))
+        assert reader.next().QUAL == None
+
 
 
 suite.addTests(unittest.TestLoader().loadTestsFromTestCase(TestGatkOutput))
@@ -292,3 +316,4 @@ suite.addTests(unittest.TestLoader().loadTestsFromTestCase(TestFilter))
 suite.addTests(unittest.TestLoader().loadTestsFromTestCase(Test1kg))
 suite.addTests(unittest.TestLoader().loadTestsFromTestCase(TestRecord))
 suite.addTests(unittest.TestLoader().loadTestsFromTestCase(TestCall))
+suite.addTests(unittest.TestLoader().loadTestsFromTestCase(TestRegression))
