@@ -9,7 +9,7 @@ specified in the meta-information lines --  specifically the ##INFO and
 against the reserved types mentioned in the spec.  Failing that, it will just
 return strings.
 
-There is currently one piece of interface: ``Reader``.  It takes a file-like
+There main interface is the class: ``Reader``.  It takes a file-like
 object and acts as a reader::
 
     >>> import vcf
@@ -19,13 +19,12 @@ object and acts as a reader::
     Record(CHROM=20, POS=14370, REF=G, ALT=['A'])
     Record(CHROM=20, POS=17330, REF=T, ALT=['A'])
     Record(CHROM=20, POS=1110696, REF=A, ALT=['G', 'T'])
-    Record(CHROM=20, POS=1230237, REF=T, ALT=['.'])
+    Record(CHROM=20, POS=1230237, REF=T, ALT=[None])
     Record(CHROM=20, POS=1234567, REF=GTCT, ALT=['G', 'GTACT'])
 
 
 This produces a great deal of information, but it is conveniently accessed.
-The attributes of a Record are the 8 fixed fields from the VCF spec plus two
-more.  That is:
+The attributes of a Record are the 8 fixed fields from the VCF spec::
 
     * ``Record.CHROM``
     * ``Record.POS``
@@ -36,13 +35,13 @@ more.  That is:
     * ``Record.FILTER``
     * ``Record.INFO``
 
-plus three more attributes to handle genotype information:
+plus attributes to handle genotype information:
 
     * ``Record.FORMAT``
     * ``Record.samples``
     * ``Record.genotype``
 
-``samples`` and ``genotypes``, not being the title of any column, is left lowercase.  The format
+``samples`` and ``genotype``, not being the title of any column, are left lowercase.  The format
 of the fixed fields is from the spec.  Comma-separated lists in the VCF are
 converted to lists.  In particular, one-entry VCF lists are converted to
 one-entry Python lists (see, e.g., ``Record.ALT``).  Semicolon-delimited lists
@@ -58,7 +57,7 @@ a ``True`` value. Integers and floats are handled exactly as you'd expect::
     >>> print record.INFO['AF']
     [0.5]
 
-There are a number of convienience functions for each ``Record`` allowing you to
+There are a number of convienience methods and properties for each ``Record`` allowing you to
 examine properties of interest::
 
     >>> print record.num_called, record.call_rate, record.num_unknown
@@ -68,7 +67,8 @@ examine properties of interest::
     >>> print record.nucl_diversity, record.aaf
     0.6 0.5
     >>> print record.get_hets()
-    [Call(sample=NA00002, GT=1|0)]
+    [Call(sample=NA00002, GT=1|0, GQ=[48])]
+
 
 ``record.FORMAT`` will be a string specifying the format of the genotype
 fields.  In case the FORMAT column does not exist, ``record.FORMAT`` is
@@ -127,15 +127,29 @@ Random access is supported for files with tabix indexes.  Simply call fetch for
 region you are interested in::
 
     >>> vcf_reader = vcf.Reader(filename='test/tb.vcf.gz')
-    >>> for record in vcf_reader.fetch('20', 1110696-1, 1230237):
+    >>> for record in vcf_reader.fetch('20', 1110696, 1230237):
     ...     print record
     Record(CHROM=20, POS=1110696, REF=A, ALT=['G', 'T'])
-    Record(CHROM=20, POS=1230237, REF=T, ALT=['.'])
+    Record(CHROM=20, POS=1230237, REF=T, ALT=[None])
+
+Or extract a single row::
+
+    >>> print vcf_reader.fetch('20', 1110696)
+    Record(CHROM=20, POS=1110696, REF=A, ALT=['G', 'T'])
+
+
+The ``Writer`` class provides a way of writing a VCF file.  Currently, you must specify a
+template ``Reader`` which provides the metadata::
+
+    >>> vcf_reader = vcf.Reader(filename='test/tb.vcf.gz')
+    >>> vcf_writer = vcf.Writer(file('/dev/null', 'w'), vcf_reader)
+    >>> for record in vcf_reader:
+    ...     vcf_writer.write_record(record)
 
 
 An extensible script is available to filter vcf files in vcf_filter.py.  VCF filters
 declared by other packages will be available for use in this script.  Please
-see FILTERS.md for full description.
+see :doc:`FILTERS` for full description.
 
 '''
 import collections
@@ -145,13 +159,15 @@ import gzip
 import sys
 import itertools
 
-
 try:
     import pysam
 except ImportError:
     pysam = None
 
 
+VERSION = '0.3.0'
+
+
 # Metadata parsers/constants
 RESERVED_INFO = {
     'AA': 'String', 'AC': 'Integer', 'AF': 'Float', 'AN': 'Integer',
@@ -173,9 +189,8 @@ _Format = collections.namedtuple('Format', ['id', 'num', 'type', 'desc'])
 
 class _vcf_metadata_parser(object):
     '''Parse the metadat in the header of a VCF file.'''
-    def __init__(self, aggressive=False):
+    def __init__(self):
         super(_vcf_metadata_parser, self).__init__()
-        self.aggro = aggressive
         self.info_pattern = re.compile(r'''\#\#INFO=<
             ID=(?P<id>[^,]+),
             Number=(?P<number>-?\d+|\.|[AG]),
@@ -199,14 +214,14 @@ class _vcf_metadata_parser(object):
         match = self.info_pattern.match(info_string)
         if not match:
             raise SyntaxError(
-                "One of the INFO lines is malformed: {}".format(info_string))
+                "One of the INFO lines is malformed: %s" % info_string)
 
         try:
             num = int(match.group('number'))
             if num < 0:
-                num = None if self.aggro else '.'
+                num = None
         except ValueError:
-            num = None if self.aggro else '.'
+            num = None
 
         info = _Info(match.group('id'), num,
                      match.group('type'), match.group('desc'))
@@ -218,8 +233,7 @@ class _vcf_metadata_parser(object):
         match = self.filter_pattern.match(filter_string)
         if not match:
             raise SyntaxError(
-                "One of the FILTER lines is malformed: {}".format(
-                    filter_string))
+                "One of the FILTER lines is malformed: %s" % filter_string)
 
         filt = _Filter(match.group('id'), match.group('desc'))
 
@@ -230,15 +244,14 @@ class _vcf_metadata_parser(object):
         match = self.format_pattern.match(format_string)
         if not match:
             raise SyntaxError(
-                "One of the FORMAT lines is malformed: {}".format(
-                    format_string))
+                "One of the FORMAT lines is malformed: %s" % format_string)
 
         try:
             num = int(match.group('number'))
             if num < 0:
-                num = None if self.aggro else '.'
+                num = None
         except ValueError:
-            num = None if self.aggro else '.'
+            num = None
 
         form = _Format(match.group('id'), num,
                        match.group('type'), match.group('desc'))
@@ -251,25 +264,33 @@ class _vcf_metadata_parser(object):
 
 
 class _Call(object):
-    """ A called genotype, an entry in a VCF file"""
-
+    """ A genotype call, a cell entry in a VCF file"""
 
     def __init__(self, site, sample, data):
+        #: The ``_Record`` for this ``_Call``
         self.site = site
+        #: The sample name
         self.sample = sample
+        #: Dictionary of data from the VCF file
         self.data = data
         self.gt_nums = self.data['GT']
-        self.called = self.gt_nums is not None and self.gt_nums != "./."
+        #: True if the GT is not ./.
+        self.called = self.gt_nums is not None
 
     def __repr__(self):
-        return "Call(sample=%s, GT=%s)" % (self.sample, self.gt_nums)
+        return "Call(sample=%s, GT=%s, GQ=%s)" % (self.sample, self.gt_nums, self.data.get('GQ', ''))
 
     def __eq__(self, other):
-        return (self.sample == other.sample and self.data == other.data)
+        """ Two _Calls are equal if their _Records are equal
+            and the samples and ``gt_type``s are the same
+        """
+        return (self.site == other.site
+                and self.sample == other.sample
+                and self.gt_type == other.gt_type)
 
     @property
     def gt_bases(self):
-        '''Return the actual genotype alleles.
+        '''The actual genotype alleles.
            E.g. if VCF genotype is 0/1, return A/G
         '''
         # nothing to do if no genotype call
@@ -289,7 +310,7 @@ class _Call(object):
 
     @property
     def gt_type(self):
-        '''Return the type of genotype.
+        '''The type of genotype.
            hom_ref  = 0
            het      = 1
            hom_alt  = 2  (we don;t track _which+ ALT)
@@ -315,18 +336,38 @@ class _Call(object):
 
     @property
     def phased(self):
-        '''Return a boolean indicating whether or not
+        '''A boolean indicating whether or not
            the genotype is phased for this sample
         '''
-        return self.data['GT'].find("|") >= 0
+        return self.data['GT'] is not None and self.data['GT'].find("|") >= 0
 
     def __getitem__(self, key):
         """ Lookup value, backwards compatibility """
         return self.data[key]
 
+    @property
+    def is_variant(self):
+        """ Return True if not a reference call """
+        if not self.called:
+            return None
+        return self.gt_type != 0
+
+    @property
+    def is_het(self):
+        """ Return True for heterozygous calls """
+        if not self.called:
+            return None
+        return self.gt_type == 1
+
 
 class _Record(object):
-    """ A set of calls at a site.  A row in a VCF file. """
+    """ A set of calls at a site.  Equivalent to a row in a VCF file.
+
+        The standard VCF fields CHROM, POS, ID, REF, ALT, QUAL, FILTER,
+        INFO and FORMAT are available as properties.
+
+        The list of genotype calls is in the ``samples`` property.
+    """
 
     def __init__(self, CHROM, POS, ID, REF, ALT, QUAL, FILTER, INFO, FORMAT, sample_indexes, samples=None):
         self.CHROM = CHROM
@@ -338,12 +379,27 @@ class _Record(object):
         self.FILTER = FILTER
         self.INFO = INFO
         self.FORMAT = FORMAT
-        # create a list of alleles. [0] = REF, [1:] = ALTS
+        #: 0-based start coordinate
+        self.start = self.POS - 1
+        #: 1-based end coordinate
+        self.end = self.start + len(self.REF)
+        #: list of alleles. [0] = REF, [1:] = ALTS
         self.alleles = [self.REF]
         self.alleles.extend(self.ALT)
+        #: list of ``_Calls`` for each sample ordered as in source VCF
         self.samples = samples
         self._sample_indexes = sample_indexes
 
+    def __eq__(self, other):
+        """ _Records are equal if they describe the same variant (same position, alleles) """
+        return (self.CHROM == other.CHROM and
+                self.POS == other.POS and
+                self.REF == other.REF and
+                self.ALT == other.ALT)
+
+    def __iter__(self):
+        return iter(self.samples)
+
     def __str__(self):
         return "Record(CHROM=%(CHROM)s, POS=%(POS)s, REF=%(REF)s, ALT=%(ALT)s)" % self.__dict__
 
@@ -351,7 +407,7 @@ class _Record(object):
         self.FORMAT = self.FORMAT + ':' + fmt
 
     def add_filter(self, flt):
-        if self.FILTER == '.' or self.FILTER == 'PASS':
+        if self.FILTER is None or self.FILTER == 'PASS':
             self.FILTER = ''
         else:
             self.FILTER = self.FILTER + ';'
@@ -361,41 +417,42 @@ class _Record(object):
         self.INFO[info] = value
 
     def genotype(self, name):
+        """ Lookup a ``_Call`` for the sample given in ``name`` """
         return self.samples[self._sample_indexes[name]]
 
     @property
     def num_called(self):
-        """Return the number of called samples"""
+        """ The number of called samples"""
         return sum(s.called for s in self.samples)
 
     @property
     def call_rate(self):
-        """ return the fraction of genotypes that were actually called. """
+        """ The fraction of genotypes that were actually called. """
         return float(self.num_called) / float(len(self.samples))
 
     @property
     def num_hom_ref(self):
-        """ return the number of homozygous for ref allele genotypes"""
+        """ The number of homozygous for ref allele genotypes"""
         return len([s for s in self.samples if s.gt_type == 0])
 
     @property
     def num_hom_alt(self):
-        """ return the number of homozygous for alt allele genotypes"""
+        """ The number of homozygous for alt allele genotypes"""
         return len([s for s in self.samples if s.gt_type == 2])
 
     @property
     def num_het(self):
-        """ return the number of heterozygous genotypes"""
+        """ The number of heterozygous genotypes"""
         return len([s for s in self.samples if s.gt_type == 1])
 
     @property
     def num_unknown(self):
-        """ return the number of unknown genotypes"""
+        """ The number of unknown genotypes"""
         return len([s for s in self.samples if s.gt_type is None])
 
     @property
     def aaf(self):
-        """Calculate the allele frequency of the alternate allele.
+        """ The allele frequency of the alternate allele.
            NOTE 1: Punt if more than one alternate allele.
            NOTE 2: Denominator calc'ed from _called_ genotypes.
         """
@@ -411,7 +468,7 @@ class _Record(object):
     @property
     def nucl_diversity(self):
         """
-        Calculate pi_hat (estimation of nucleotide diversity) for the site.
+        pi_hat (estimation of nucleotide diversity) for the site.
         This metric can be summed across multiple sites to compute regional
         nucleotide diversity estimates.  For example, pi_hat for all variants
         in a given gene.
@@ -429,25 +486,33 @@ class _Record(object):
         return float(num_chroms/(num_chroms-1.0)) * (2.0 * p * q)
 
     def get_hom_refs(self):
-        """ return the list of hom ref genotypes"""
+        """ The list of hom ref genotypes"""
         return [s for s in self.samples if s.gt_type == 0]
 
     def get_hom_alts(self):
-        """ return the list of hom alt genotypes"""
+        """ The list of hom alt genotypes"""
         return [s for s in self.samples if s.gt_type == 2]
 
     def get_hets(self):
-        """ return the list of het genotypes"""
+        """ The list of het genotypes"""
         return [s for s in self.samples if s.gt_type == 1]
 
     def get_unknowns(self):
-        """ return the list of unknown genotypes"""
+        """ The list of unknown genotypes"""
         return [s for s in self.samples if s.gt_type is None]
 
 
 class Reader(object):
-    '''Read and parse a VCF v 4.0 file'''
-    def __init__(self, fsock=None, filename=None, aggressive=False, compressed=False):
+    """ Reader for a VCF v 4.0 file, an iterator returning ``_Record objects`` """
+
+
+    def __init__(self, fsock=None, filename=None, compressed=False, prepend_chr=False):
+        """ Create a new Reader for a VCF file.
+
+            You must specify either fsock (stream) or filename.  Gzipped streams
+            or files are attempted to be recogized by the file extension, or gzipped
+            can be forced with ``compressed=True``
+        """
         super(VCFReader, self).__init__()
 
         if not (fsock or filename):
@@ -468,19 +533,19 @@ class Reader(object):
         if compressed or (filename and filename.endswith('.gz')):
             self.reader = gzip.GzipFile(fileobj=self.reader)
 
-        self.aggro = aggressive
+        #: metadata fields from header
         self.metadata = None
+        #: INFO fields from header
         self.infos = None
+        #: FILTER fields from header
         self.filters = None
+        #: FORMAT fields from header
         self.formats = None
         self.samples = None
         self._sample_indexes = None
         self._header_lines = []
         self._tabix = None
-        if aggressive:
-            self._mapper = self._none_map
-        else:
-            self._mapper = self._pass_map
+        self._prepend_chr = prepend_chr
         self._parse_metainfo()
 
     def __iter__(self):
@@ -523,16 +588,11 @@ class Reader(object):
         self.samples = fields[9:]
         self._sample_indexes = dict([(x,i) for (i,x) in enumerate(self.samples)])
 
-    def _none_map(self, func, iterable, bad='.'):
+    def _map(self, func, iterable, bad='.'):
         '''``map``, but make bad values None.'''
         return [func(x) if x != bad else None
                 for x in iterable]
 
-    def _pass_map(self, func, iterable, bad='.'):
-        '''``map``, but make bad values None.'''
-        return [func(x) if x != bad else bad
-                for x in iterable]
-
     def _parse_info(self, info_str):
         '''Parse the INFO field of a VCF entry into a dictionary of Python
         types.
@@ -556,10 +616,10 @@ class Reader(object):
 
             if entry_type == 'Integer':
                 vals = entry[1].split(',')
-                val = self._mapper(int, vals)
+                val = self._map(int, vals)
             elif entry_type == 'Float':
                 vals = entry[1].split(',')
-                val = self._mapper(float, vals)
+                val = self._map(float, vals)
             elif entry_type == 'Flag':
                 val = True
             elif entry_type == 'String':
@@ -600,8 +660,6 @@ class Reader(object):
         return samp_data
 
     def _parse_sample(self, sample, samp_fmt, samp_fmt_types):
-        mapper = self._mapper
-
         sampdict = dict([(x, None) for x in samp_fmt])
 
         for fmt, entry_type, vals in itertools.izip(samp_fmt, samp_fmt_types, sample.split(':')):
@@ -612,17 +670,15 @@ class Reader(object):
                 gt = vals[0]
 
                 if gt == './.':
-                    if self.aggro:
-                        gt = None
-                    sampdict[fmt] = gt
+                    sampdict[fmt] = None
                     break
                 else:
                     sampdict[fmt] = gt
             else:
                 if entry_type == 'Integer':
-                    sampdict[fmt] = mapper(int, vals)
+                    sampdict[fmt] = self._map(int, vals)
                 elif entry_type == 'Float' or entry_type == 'Numeric':
-                    sampdict[fmt] = mapper(float, vals)
+                    sampdict[fmt] = self._map(float, vals)
                 else:
                     sampdict[fmt] = vals
         return sampdict
@@ -631,22 +687,24 @@ class Reader(object):
         '''Return the next record in the file.'''
         row = self.reader.next().split()
         chrom = row[0]
+        if self._prepend_chr:
+            chrom = 'chr' + chrom
         pos = int(row[1])
 
         if row[2] != '.':
             ID = row[2]
         else:
-            ID = None if self.aggro else row[2]
+            ID = None
 
         ref = row[3]
-        alt = self._mapper(str, row[4].split(','))
+        alt = self._map(str, row[4].split(','))
 
         if row[5] == '.':
-            qual = '.'
+            qual = None
         else:
             qual = float(row[5]) if '.' in row[5] else int(row[5])
         filt = row[6].split(';') if ';' in row[6] else row[6]
-        if filt == 'PASS' and self.aggro:
+        if filt == 'PASS':
             filt = None
         info = self._parse_info(row[7])
 
@@ -663,7 +721,11 @@ class Reader(object):
 
         return record
 
-    def fetch(self, chrom, start, end):
+    def fetch(self, chrom, start, end=None):
+        """ fetch records from a Tabix indexed VCF, requires pysam
+            if start and end are specified, return iterator over positions
+            if end not specified, return individual ``_Call`` at start or None
+        """
         if not pysam:
             raise Exception('pysam not available, try "pip install pysam"?')
 
@@ -673,11 +735,25 @@ class Reader(object):
         if not self._tabix:
             self._tabix = pysam.Tabixfile(self.filename)
 
+        if self._prepend_chr and chrom[:3] == 'chr':
+            chrom = chrom[3:]
+
+        # not sure why tabix needs position -1
+        start = start - 1
+
+        if end is None:
+            self.reader = self._tabix.fetch(chrom, start, start+1)
+            try:
+                return self.next()
+            except StopIteration:
+                return None
+
         self.reader = self._tabix.fetch(chrom, start, end)
         return self
 
 
 class Writer(object):
+    """ VCF Writer """
 
     fixed_fields = "#CHROM POS ID REF ALT QUAL FILTER INFO FORMAT".split()
 
@@ -688,80 +764,76 @@ class Writer(object):
         for line in template.metadata.items():
             stream.write('##%s=%s\n' % line)
         for line in template.infos.values():
-            stream.write('##INFO=<ID=%s,Number=%s,Type=%s,Description="%s">\n' % line)
+            stream.write('##INFO=<ID=%s,Number=%s,Type=%s,Description="%s">\n' % tuple(self._map(str, line)))
         for line in template.formats.values():
-            stream.write('##FORMAT=<ID=%s,Number=%s,Type=%s,Description="%s">\n' % line)
-
+            stream.write('##FORMAT=<ID=%s,Number=%s,Type=%s,Description="%s">\n' % tuple(self._map(str, line)))
         for line in template.filters.values():
-            stream.write('##FILTER=<ID=%s,Description="%s">\n' % line)
+            stream.write('##FILTER=<ID=%s,Description="%s">\n' % tuple(self._map(str, line)))
 
-        self.info_pattern = re.compile(r'''\#\#INFO=<
-            ID=(?P<id>[^,]+),
-            Number=(?P<number>-?\d+|\.|[AG]),
-            Type=(?P<type>Integer|Float|Flag|Character|String),
-            Description="(?P<desc>[^"]*)"
-            >''', re.VERBOSE)
-        self.filter_pattern = re.compile(r'''\#\#FILTER=<
-            ID=(?P<id>[^,]+),
-            Description="(?P<desc>[^"]*)"
-            >''', re.VERBOSE)
-        self.format_pattern = re.compile(r'''\#\#FORMAT=<
-            ID=(?P<id>.+),
-            Number=(?P<number>-?\d+|\.|[AG]),
-            Type=(?P<type>.+),
-            Description="(?P<desc>.*)"
-            >''', re.VERBOSE)
-        self.meta_pattern = re.compile(r'''##(?P<key>.+)=(?P<val>.+)''')
+        self._write_header()
 
-        self.write_header()
-
-    def write_header(self):
+    def _write_header(self):
         # TODO: write INFO, etc
         self.writer.writerow(self.fixed_fields + self.template.samples)
 
     def write_record(self, record):
-        ffs = [record.CHROM, record.POS, record.ID, record.REF, self._format_alt(record.ALT),
-            record.QUAL, record.FILTER, self._format_info(record.INFO), record.FORMAT]
+        """ write a record to the file """
+        ffs = self._map(str, [record.CHROM, record.POS, record.ID, record.REF]) \
+              + [self._format_alt(record.ALT), record.QUAL, record.FILTER or '.',
+                 self._format_info(record.INFO), record.FORMAT]
 
         samples = [self._format_sample(record.FORMAT, sample)
             for sample in record.samples]
         self.writer.writerow(ffs + samples)
 
     def _format_alt(self, alt):
-        return ','.join(alt)
+        return ','.join([x or '.' for x in alt])
 
     def _format_info(self, info):
         return ';'.join(["%s=%s" % (x, self._stringify(y)) for x, y in info.items()])
 
     def _format_sample(self, fmt, sample):
-        if sample.data["GT"] == "./.":
+        if sample.data["GT"] is None:
             return "./."
-        return ':'.join((str(self._stringify(sample.data[f])) for f in fmt.split(':')))
+        return ':'.join(self._stringify(sample.data[f]) for f in fmt.split(':'))
 
-    def _stringify(self, x):
+    def _stringify(self, x, none='.'):
         if type(x) == type([]):
-            return ','.join(map(str, x))
-        return str(x)
+            return ','.join(self._map(str, x, none))
+        return str(x) if x is not None else none
 
+    def _map(self, func, iterable, none='.'):
+        '''``map``, but make None values none.'''
+        return [func(x) if x is not None else none
+                for x in iterable]
 
 
 class Filter(object):
-    name = 'filter'
+    """ Base class for vcf_filter.py filters """
+
+    name = 'f'
+    """ name used to activate filter and in VCF headers """
+
     description = 'VCF filter base class'
-    short_name = 'f'
+    """ descrtiption used in vcf headers """
 
     @classmethod
-    def customize_parser(self):
+    def customize_parser(self, parser):
+        """ hook to extend argparse parser with custom arguments """
         pass
 
     def __init__(self, args):
+        """ create the filter using argparse ``args`` """
         self.threshold = 0
 
     def __call__(self):
+        """ filter a site, return not None if the site should be filtered """
         raise NotImplementedError('Filters must implement this method')
 
+
     def filter_name(self):
-        return '%s%s' % (self.short_name, self.threshold)
+        """ return the name to put in the VCF header, default is ``name`` + ``threshold`` """
+        return '%s%s' % (self.name, self.threshold)
 
 
 def __update_readme():
@@ -772,4 +844,3 @@ def __update_readme():
 # backwards compatibility
 VCFReader = Reader
 VCFWriter = Writer
-
